from collections.abc import AsyncGenerator, Generator, Mapping
from typing import TYPE_CHECKING, Any, Generic, Type, TypeAlias, TypeVar

from motor.motor_asyncio import AsyncIOMotorCursor
from pydantic import BaseModel
from pymongo.results import DeleteResult

from mango.expression import Expression, ExpressionField
from mango.index import Order
from mango.utils import any_check, field_validate, is_sequence

if TYPE_CHECKING:  # pragma: no cover
    from mango.models import Model

T_Model = TypeVar("T_Model", bound="Model")

KeyField: TypeAlias = str | ExpressionField

FindMapping: TypeAlias = Mapping[KeyField, Any]

DirectionType: TypeAlias = Order

SortType: TypeAlias = tuple[str, DirectionType]


class FindOptions(BaseModel):
    limit: int = 0
    skip: int = 0
    sort: list[SortType] = []

    def kwdict(self, *exclude: str) -> dict[str, Any]:
        return self.dict(exclude=set(exclude), exclude_defaults=True)


class FindResult(Generic[T_Model]):
    def __init__(
        self,
        model: Type[T_Model],
        *filter: FindMapping | Expression,
    ) -> None:
        self.model = model
        self.collection = model.meta.collection
        self._filter = filter
        self.options = FindOptions()

    def __await__(self) -> Generator[None, None, list[T_Model]]:
        """
        `await` : 等待时，将返回获取的模型列表
        """
        documents = yield from self.cursor.to_list(length=None).__await__()
        instances: list[T_Model] = []
        for document in documents:
            instances.append(self.model.from_doc(document))
            yield
        return instances

    async def __aiter__(self) -> AsyncGenerator[T_Model, None]:
        """`async for`: 异步迭代查询结果"""
        async for document in self.cursor:
            yield self.model.from_doc(document)

    @property
    def cursor(self) -> AsyncIOMotorCursor:
        return self.collection.find(self.filter, **self.options.kwdict())

    @property
    def filter(self) -> dict[str, Any]:
        """查询过滤条件"""
        compiled: dict[str, Any] = {}
        for condition in self._filter:
            if isinstance(condition, Mapping):
                condition = dict(condition)
            elif isinstance(condition, Expression):
                condition = condition.struct()
            else:
                raise RuntimeError("查询过滤条件不正确, 应为映射或表达式")
            compiled |= self._compile(condition)
        return compiled

    def _compile(
        self,
        source: Mapping[KeyField, Any] | Mapping[str, Any],
    ) -> dict[str, Any]:
        compiled: dict[str, Any] = {}

        for key, value in source.items():
            key = str(key)
            if isinstance(value, Expression):
                compiled[key] = value.struct()
            elif isinstance(value, Mapping):
                compiled[key] = self._compile(value)
            elif is_sequence(value):
                compiled[key] = []
                for i in value:
                    if isinstance(i, Expression):
                        i = i.struct()
                    if isinstance(i, Mapping):
                        i = self._compile(i)
                    compiled[key].append(i)
            else:
                compiled[key] = value

        return compiled

    def limit(self, limit: int = 0) -> "FindResult[T_Model]":
        """限制查询条件返回结果的数量"""
        self.options.limit += limit
        return self

    def skip(self, skip: int = 0) -> "FindResult[T_Model]":
        """跳过指定数目的文档"""
        self.options.skip += skip
        return self

    def sort(self, *orders: Any) -> "FindResult[T_Model]":
        """对查询文档流进行排序"""
        if not (len(orders) != 2 or any_check(orders, is_sequence)):
            orders = (orders,)
        for order in orders:
            direction = Order.ASC
            if is_sequence(order):
                key, direction = order
            else:
                key = order

            try:
                key, direction = str(key), Order(direction)
            except ValueError as e:
                raise RuntimeError("键应为字符串或字段, 排序方向应为 Order 枚举成员") from e
            else:
                self.options.sort.append((key, direction))

        return self

    def asc(self, *keys: Any) -> "FindResult[T_Model]":
        """升序排列"""
        self.options.sort.extend(self._sort_order(*keys))
        return self

    def desc(self, *keys: Any) -> "FindResult[T_Model]":
        """降序排列"""
        self.options.sort.extend(self._sort_order(*keys, reverse=True))
        return self

    def _sort_order(
        self, *keys: Any, reverse: bool = False
    ) -> Generator[SortType, None, None]:
        direction: Order = Order.DESC if reverse else Order.ASC
        for key in keys:
            yield str(key), direction

    async def count(self) -> int:
        """获得符合条件的文档总数"""
        return await self.collection.count_documents(
            self.filter, **self.options.kwdict("sort")
        )

    async def get(self) -> T_Model | None:
        """
        从数据库中获取单个文档。
        返回单个文档，如果没有找到匹配的文档，返回“None”。
        """
        if document := await self.collection.find_one(self.filter):
            return self.model.from_doc(document)

    async def get_or_create(
        self, defaults: dict[str, Any] | T_Model | None = None
    ) -> T_Model:
        """获取文档, 如果不存在, 则创建"""
        if model := await self.get():
            return model
        if isinstance(defaults, dict | None):
            defaults = self.model(**(defaults or {}))
        return await defaults.save()

    async def delete(self) -> int:
        """删除符合条件的文档"""
        result: DeleteResult = await self.collection.delete_many(self.filter)
        return result.deleted_count

    async def update(self, **kwargs: Any) -> None:
        """使用提供的信息更新查找到的文档"""
        values = field_validate(self.model, kwargs)
        await self.collection.update_many(self.filter, {"$set": values})



from dataclasses import dataclass
from enum import Enum, auto
from typing import TYPE_CHECKING, Any

from pydantic.fields import ModelField
from typing_extensions import Self

if TYPE_CHECKING:  # pragma: no cover
    from mango.models import Model


class Operators(Enum):
    EQ = auto()
    """等于"""
    NE = auto()
    """不等于"""
    LT = auto()
    """小于"""
    LTE = auto()
    """小于或等于"""
    GT = auto()
    """大于"""
    GTE = auto()
    """大于或等于"""
    OR = auto()
    """逻辑或"""
    AND = auto()
    """逻辑与"""
    NOR = auto()
    """逻辑非或"""
    IN = auto()
    """包含在"""
    NIN = auto()
    """不包含在"""
    REGEX = auto()
    """正则匹配"""

    def __str__(self) -> str:
        return f"${self.name.lower()}"


class ExpressionField:
    def __init__(self, field: ModelField, parents: list[tuple[str, "Model"]]) -> None:
        self.field = field
        self.parents = parents

    def __eq__(self, other: Any) -> "Expression":
        return OPR(self).eq(other)

    def __ne__(self, other: Any) -> "Expression":
        return OPR(self).ne(other)

    def __lt__(self, other: Any) -> "Expression":
        return OPR(self).lt(other)

    def __le__(self, other: Any) -> "Expression":
        return OPR(self).lte(other)

    def __gt__(self, other: Any) -> "Expression":
        return OPR(self).gt(other)

    def __ge__(self, other: Any) -> "Expression":
        return OPR(self).gte(other)

    def __hash__(self) -> int:
        return super().__hash__()

    def __repr__(self) -> str:
        return f"ExpressionField(name={str(self)}, type={self.field.type_.__name__})"

    def __str__(self) -> str:
        names = [p[0] for p in self.parents]
        names.append(self.field.name)
        return ".".join(names)

    def __getattr__(self, name: str) -> Any:
        """内嵌查询反向查找分配父级"""
        attr = getattr(self.field.outer_type_, name)
        if isinstance(attr, self.__class__):
            new_parent = (self.field.name, self.field.outer_type_)
            if new_parent not in attr.parents:
                attr.parents.append(new_parent)
            if new_parents := list(set(self.parents) - set(attr.parents)):
                attr.parents = new_parents + attr.parents
        return attr


@dataclass
class Expression:
    key: ExpressionField | None
    operator: Operators
    value: Any

    def __or__(self, other: Self) -> Self:
        return OPR.or_(self, other)

    def __and__(self, other: Self) -> Self:
        return OPR.and_(self, other)

    def __repr__(self) -> str:
        return f"Expression({self.struct()})"

    def struct(self) -> dict[str, Any]:
        """转换为 MongoDB 查询结构"""
        value = {str(self.operator): self.unpack(self.value)}
        return {str(self.key): value} if self.key else value

    def unpack(self, value: Any):
        # TODO: 将嵌入文档模型转换为 mongodb 文档形式
        return value

    def merge(self, operator: Operators):
        return self.value if self.operator is operator else [self]


class OPR:
    def __init__(self, key: Any) -> None:
        if not isinstance(key, ExpressionField):
            raise RuntimeError
        self.key = key

    def eq(self, value: Any) -> Expression:
        return Expression(self.key, Operators.EQ, value)

    def ne(self, value: Any) -> Expression:
        return Expression(self.key, Operators.NE, value)

    def lt(self, value: Any) -> Expression:
        return Expression(self.key, Operators.LT, value)

    def lte(self, value: Any) -> Expression:
        return Expression(self.key, Operators.LTE, value)

    def gt(self, value: Any) -> Expression:
        return Expression(self.key, Operators.GT, value)

    def gte(self, value: Any) -> Expression:
        return Expression(self.key, Operators.GTE, value)

    def in_(self, *values: Any) -> Expression:
        """包含在"""
        return Expression(self.key, Operators.IN, values)

    def nin(self, *values: Any) -> Expression:
        """不包含在"""
        return Expression(self.key, Operators.NIN, values)

    def regex(self, values: str) -> Expression:
        """正则匹配"""
        return Expression(self.key, Operators.REGEX, values)

    @classmethod
    def or_(cls, *expressions: Expression | bool) -> Expression:
        merge = cls._merge(Operators.OR, expressions)
        return Expression(None, *merge)

    @classmethod
    def and_(cls, *expressions: Expression | bool) -> Expression:
        merge = cls._merge(Operators.AND, expressions)
        return Expression(None, *merge)

    @classmethod
    def nor(cls, *expressions: Expression | bool) -> Expression:
        """既不……也不……"""
        merge = cls._merge(Operators.NOR, expressions)
        return Expression(None, *merge)

    @classmethod
    def _merge(
        cls, operator: Operators, expressions: tuple[Expression | bool]
    ) -> tuple[Operators, list[Expression]]:
        merge_expr: list[Expression] = []
        for expression in expressions:
            if not isinstance(expression, Expression):
                raise RuntimeError("必须是有效的表达式")

            if expression.operator is operator:
                merge_expr.extend(expression.value)
            else:
                merge_expr.append(expression)

        return operator, merge_expr



from mango.drive import connect
from mango.expression import OPR
from mango.fields import Field
from mango.index import Attr, Index, Order
from mango.models import Model

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:  # pragma: no cover
    import importlib_metadata  # type: ignore

__version__ = importlib_metadata.version(__name__)

__all__ = ["connect", "OPR", "Field", "Attr", "Index", "Order", "Model"]



import re
from collections.abc import Callable, Iterable, Sequence
from types import UnionType
from typing import TYPE_CHECKING, Any, Type

import pydantic

if TYPE_CHECKING:  # pragma: no cover
    from mango.models import Model


def to_snake_case(string: str) -> str:
    """将字符串转换为蛇形命名法"""
    tmp = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", string)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", tmp).lower()


def all_check(
    iter_obj: Iterable[object],
    type_or_func: type
    | UnionType
    | Callable
    | tuple[type | UnionType | tuple[Any, ...], ...],
) -> bool:
    """
    如果可迭代对象中的所有元素为指定类型，则返回True。
    如果可迭代对象为空，则返回True。
    """
    if isinstance(type_or_func, Callable):
        return all(type_or_func(obj) for obj in iter_obj)
    else:
        return all(isinstance(obj, type_or_func) for obj in iter_obj)


def any_check(
    iter_obj: Iterable[object],
    type_or_func: type
    | UnionType
    | Callable
    | tuple[type | UnionType | tuple[Any, ...], ...],
) -> bool:
    """
    如果可迭代对象中的任意元素为指定类型，则返回True。
    如果可迭代对象为空，则返回False。
    """
    if isinstance(type_or_func, Callable):
        return any(type_or_func(obj) for obj in iter_obj)
    else:
        return any(isinstance(obj, type_or_func) for obj in iter_obj)


def is_sequence(
    iter_obj: Sequence[object],
) -> bool:
    """判断是否为非字符串的序列对象"""
    return isinstance(iter_obj, Sequence) and not isinstance(iter_obj, bytes | str)


def field_validate(model: Type["Model"], input_data: dict[str, Any]) -> dict[str, Any]:
    """验证模型的指定字段"""
    if miss := set(input_data) - set(model.__fields__):
        raise ValueError(f"这些字段在 {model.__name__} 中不存在: {miss}")

    fields = {k: v for k, v in model.__fields__.items() if k in input_data}
    new_model = pydantic.create_model(model.__name__, **fields)
    values, _, validation_error = pydantic.validate_model(new_model, input_data)

    if validation_error:
        raise validation_error

    return values



from collections.abc import Generator, Mapping, Sequence
from typing import TYPE_CHECKING, Any, ClassVar

from bson import ObjectId
from pydantic import BaseModel
from pydantic.fields import ModelField
from pydantic.main import ModelMetaclass
from pymongo.results import DeleteResult
from typing_extensions import Self, dataclass_transform

from mango.drive import Client, Collection, Database
from mango.expression import Expression, ExpressionField
from mango.fields import Field, FieldInfo, ObjectIdField
from mango.index import Index, IndexType
from mango.result import FindMapping, FindResult
from mango.utils import all_check, field_validate, to_snake_case


class MetaData(BaseModel):
    database: Database
    collection: Collection
    primary_key: str
    indexes: Sequence[Index]

    class Config:
        arbitrary_types_allowed = True
        frozen = True


@dataclass_transform(kw_only_default=True, field_specifiers=(Field, FieldInfo))
class ModelMeta(ModelMetaclass):
    if TYPE_CHECKING:  # pragma: no cover
        __fields__: dict[str, ModelField]
        meta: MetaData

    def __new__(
        cls,
        cname: str,
        bases: tuple[type, ...],
        attrs: dict[str, Any],
        *,
        name: str | None = None,
        db: Database | str | None = None,
        embedded: bool = False,
        indexes: Sequence[Index] | None = None,
    ):
        meta: dict[str, Any] = {}

        base, *_ = bases
        if not (embedded or base is BaseModel):
            meta["database"] = cls.__get_database(cls, db)
            meta["collection"] = meta["database"][name or to_snake_case(cname)]
            meta["primary_key"] = cls.__get_default_pk(cls, bases, attrs)
            meta["indexes"] = [*(indexes or []), *(cls.__get_indexes(cls, attrs))]

        scls = super().__new__(cls, cname, bases, attrs)

        if raw_meta := attrs.pop("Meta", None) or getattr(scls, "meta", None):
            meta = raw_meta.dict() | meta

        # 由于此处代码使用了 setattr，导致子类重写父类的字段时会引发错误，暂无解决办法
        # NameError: Field name "id" shadows a BaseModel attribute; use a different field name with "alias='id'".
        for field_name, field in scls.__fields__.items():
            setattr(scls, field_name, ExpressionField(field, []))
            if meta.get("primary_key"):
                continue
            if isinstance(field.field_info, FieldInfo) and field.field_info.primary_key:
                meta["primary_key"] = field.field_info.alias or field_name

        if meta:
            scls.meta = MetaData(**meta)

        return scls

    def __get_default_pk(self, bases, attrs: dict[str, Any]) -> str:
        for base in bases:
            if getattr(base, "id", None):
                return ""

        if "id" in attrs["__annotations__"]:
            return ""

        attrs["id"] = Field(default_factory=ObjectId, init=False)
        attrs["__annotations__"]["id"] = ObjectIdField
        return "id"

    def __get_database(self, db: Database | str | None = None) -> Database:
        try:
            client = Client._client[0]
        except IndexError as e:
            raise RuntimeError("无客户端连接") from e

        if isinstance(db, Database):
            return db
        elif isinstance(db, str):
            return client[db]
        else:
            return client.default_database

    def __get_indexes(self, attrs: dict[str, Any]) -> Generator[Index, None, None]:
        for name, attr in attrs.items():
            if isinstance(attr, FieldInfo):
                if index := attr.index:
                    if index is True:
                        yield Index(name)
                    elif isinstance(index, IndexType):
                        yield Index((name, index))
                    elif isinstance(index, Index):
                        yield index
                elif expire := attr.expire:
                    yield Index(name, expireAfterSeconds=expire)


class Model(BaseModel, metaclass=ModelMeta):
    id: ClassVar[ObjectId]
    meta: ClassVar[MetaData]

    if TYPE_CHECKING:  # pragma: no cover

        def __init_subclass__(
            cls,
            *,
            name: str | None = None,
            db: Database | str | None = None,
            embedded: bool = False,
            indexes: Sequence[Index] | None = None,
        ) -> None:
            ...

    @property
    def pk(self) -> Any:
        """主键值"""
        return getattr(self, self.meta.primary_key)

    async def save(self) -> Self:
        """保存数据"""
        await self.meta.collection.insert_one(self.doc())
        return self

    async def update(self, **kwargs) -> Self:
        """更新数据"""
        if kwargs:
            values = field_validate(self.__class__, kwargs)
            self = self.copy(update=values)
        await self.meta.collection.update_one(
            {"_id": self.pk}, {"$set": self.dict(exclude={self.meta.primary_key})}
        )
        return self

    async def delete(self) -> bool:
        """删除数据"""
        result: DeleteResult = await self.meta.collection.delete_one({"_id": self.pk})
        return bool(result.deleted_count)

    async def aggregate(self, pipeline: Sequence[Mapping[str, Any]]):
        """聚合查询"""
        ...

    def doc(self) -> dict[str, Any]:
        """转换为 MongoDB 文档"""
        data = self.dict()
        data["_id"] = data.pop(self.meta.primary_key)
        return data

    @classmethod
    def from_doc(cls, document: dict[str, Any]) -> Self:
        """从文档构建模型实例"""
        document[cls.meta.primary_key] = document.pop("_id")
        return cls(**document)

    @classmethod
    def find(
        cls,
        *args: FindMapping | Expression | bool,
    ) -> FindResult[Self]:
        """使用表达式查询数据"""
        if all_check(args, Expression | Mapping):
            return FindResult(cls, *args)  # type: ignore
        else:
            raise RuntimeError("查询表达式类型不正确")

    @classmethod
    async def get(cls, _id: Any) -> Self | None:
        """通过主键查询数据"""
        return await cls.find({"_id": _id}).get()

    class Config:
        validate_assignment = True



from collections.abc import Mapping
from enum import Enum, unique
from typing import Any, TypeAlias

import pymongo


class IndexEnum(Enum):
    def __str__(self) -> str:
        return self.name


@unique
class Order(int, IndexEnum):
    ASC = pymongo.ASCENDING
    """升序排序"""
    DESC = pymongo.DESCENDING
    """降序排序"""


@unique
class Attr(str, IndexEnum):
    GEO2D = pymongo.GEO2D
    """二维地理空间索引"""
    GEOSPHERE = pymongo.GEOSPHERE
    """球型地理空间索引"""
    HASHED = pymongo.HASHED
    """散列索引"""
    TEXT = pymongo.TEXT
    """文本索引"""


IndexType: TypeAlias = Order | Attr
IndexTuple: TypeAlias = tuple[str, IndexType | Mapping[str, Any]]


class Index(pymongo.IndexModel):
    ASC = Order.ASC
    """升序排序"""
    DESC = Order.DESC
    """降序排序"""
    GEO2D = Attr.GEO2D
    """二维地理空间索引"""
    GEOSPHERE = Attr.GEOSPHERE
    """球型地理空间索引"""
    HASHED = Attr.HASHED
    """散列索引"""
    TEXT = Attr.TEXT
    """文本索引"""

    def __init__(
        self,
        *key: str | IndexTuple,
        name: str | None = None,
        unique: bool = False,
        background: bool = False,
        sparse: bool = False,
        **kwargs,
    ) -> None:
        key = tuple((k, self.ASC) if isinstance(k, str) else k for k in key)
        super().__init__(
            key,
            name=name,
            unique=unique,
            background=background,
            sparse=sparse,
            **kwargs,
        )



from collections.abc import Callable, Generator, Mapping
from typing import AbstractSet, Any, AnyStr

from bson import ObjectId
from pydantic.fields import FieldInfo as PDFieldInfo
from pydantic.fields import Undefined
from pydantic.typing import NoArgAnyCallable
from typing_extensions import Self

from mango.index import Index


class ObjectIdField(ObjectId):
    @classmethod
    def __get_validators__(cls) -> Generator[Callable[[AnyStr], Self], None, None]:
        yield cls.validate

    @classmethod
    def validate(cls, v: AnyStr) -> Self:
        if cls.is_valid(v):
            return cls(v)
        raise ValueError("无效的 ObjectId")

    @classmethod
    def __modify_schema__(cls, field_schema: dict) -> None:
        field_schema.update(type="string")


class FieldInfo(PDFieldInfo):
    def __init__(self, default: Any = Undefined, **kwargs: Any) -> None:
        self.primary_key: bool = kwargs.pop("primary_key", False)
        self.index: bool | str | int | Index | None = kwargs.pop("index", None)
        self.expire: int | None = kwargs.pop("expire", None)
        super().__init__(default=default, **kwargs)


def Field(
    default: Any = Undefined,
    *,
    default_factory: NoArgAnyCallable | None = None,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    exclude: AbstractSet[int | str] | Mapping[int | str, Any] | Any | None = None,
    include: AbstractSet[int | str] | Mapping[int | str, Any] | Any | None = None,
    const: bool | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    multiple_of: float | None = None,
    max_digits: int | None = None,
    decimal_places: int | None = None,
    min_items: int | None = None,
    max_items: int | None = None,
    unique_items: bool | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    allow_mutation: bool = True,
    regex: str | None = None,
    discriminator: str | None = None,
    repr: bool = True,
    primary_key: bool = False,
    index: bool | str | int | Index | None = None,
    expire: int | None = None,
    unique: bool = False,
    **extra: Any,
) -> Any:
    """
    primary_key: 主键
    index: 索引
    expire: 到期时间, int 表示创建后多少秒后到期, datetime 表示到期时间
    unique: 唯一索引
    """
    field_info = FieldInfo(
        default,
        default_factory=default_factory,
        alias=alias,
        title=title,
        description=description,
        exclude=exclude,
        include=include,
        const=const,
        gt=gt,
        ge=ge,
        lt=lt,
        le=le,
        multiple_of=multiple_of,
        max_digits=max_digits,
        decimal_places=decimal_places,
        min_items=min_items,
        max_items=max_items,
        unique_items=unique_items,
        min_length=min_length,
        max_length=max_length,
        allow_mutation=allow_mutation,
        regex=regex,
        discriminator=discriminator,
        repr=repr,
        primary_key=primary_key,
        index=index,
        expire=expire,
        unique=unique,
        **extra,
    )
    field_info._validate()
    return field_info



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mango']

package_data = \
{'': ['*']}

modules = \
['py']
install_requires = \
['motor>=3.0.0,<4.0.0', 'pydantic>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'mango-odm',
    'version': '0.1.0',
    'description': '🥭 Async MongoDB ODM with type hints in Python',
    'long_description': '# Mango\n',
    'author': 'Akirami',
    'author_email': 'Akiramiaya@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/A-kirami/mango',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from collections.abc import Iterator
from typing import Any

from motor.motor_asyncio import (
    AsyncIOMotorClient,
    AsyncIOMotorCollection,
    AsyncIOMotorDatabase,
)
from typing_extensions import Self


class Collection:
    def __init__(self, collection: AsyncIOMotorCollection) -> None:
        self.collection = collection

    def __getattr__(self, name: str) -> Any:
        return getattr(self.collection, name)

    def __repr__(self) -> str:
        return f"Collection(name={self.name}, db={self.full_name.split('.')[0]})"

    @property
    def name(self) -> str:
        return self.collection.name

    @property
    def full_name(self) -> str:
        return self.collection.full_name


class Database:
    def __init__(self, db: AsyncIOMotorDatabase) -> None:
        self.db = db
        self.collections: dict[str, Collection] = {}

    def __getattr__(self, name) -> Collection:
        try:
            return self.collections[name]
        except KeyError:
            collection: AsyncIOMotorCollection = self.db.get_collection(name)
            self.collections[name] = Collection(collection)
            return self.collections[name]

    def __getitem__(self, name: str) -> Collection:
        return self.__getattr__(name)

    def __iter__(self) -> Iterator[Collection]:
        return iter(self.collections.values())

    def __repr__(self) -> str:
        return f"Database(name={self.name}, host={self.client.HOST}, port={self.client.PORT})"

    async def drop_collection(self, collection: str | Collection):
        """删除集合"""
        name = collection if isinstance(collection, str) else collection.name
        await self.db.drop_collection(name)
        self.collections.pop(name, None)

    @property
    def name(self) -> str:
        return self.db.name

    @property
    def client(self) -> AsyncIOMotorClient:
        return self.db.client


class Client:
    _client: list[Self] = []

    def __init__(self, uri: str = "mongodb://localhost:27017", **kwargs) -> None:
        self.client = AsyncIOMotorClient(uri, **kwargs)
        self.databases: dict[str, Database] = {}
        self.__class__._client.append(self)

    def __getattr__(self, name) -> Database:
        try:
            return self.databases[name]
        except KeyError:
            db: AsyncIOMotorDatabase = self.client.get_database(name)
            self.databases[name] = Database(db)
            return self.databases[name]

    def __getitem__(self, name: str) -> Database:
        return self.__getattr__(name)

    def __iter__(self) -> Iterator[Database]:
        return iter(self.databases.values())

    def __repr__(self) -> str:
        return f"Client(host={self.host}, port={self.port})"

    async def drop_database(self, database: str | Database):
        """删除数据库"""
        name = database if isinstance(database, str) else database.name
        await self.client.drop_database(name)
        self.databases.pop(name, None)

    @property
    def default_database(self) -> Database:
        """默认为首次调用的数据库, 如果不存在, 则创建 test 数据库"""
        try:
            return next(iter(self.databases.values()))
        except StopIteration:
            return self.test

    @property
    def host(self) -> str:
        return self.client.HOST

    @property
    def port(self) -> int:
        return self.client.PORT

    @property
    def address(self) -> tuple[str, int]:
        return self.client.address


def connect(
    db: str | None = None, /, uri: str = "mongodb://localhost:27017", **kwargs
) -> Client:
    client = Client(uri, **kwargs)
    db_name: AsyncIOMotorDatabase = client.client.get_default_database("test").name
    getattr(client, db or db_name)
    return client
